import random

from django.contrib.auth.mixins import UserPassesTestMixin
from django.forms import ModelForm, ImageField
from django.urls import reverse

from ckeditor.fields import RichTextField
from filer.models.imagemodels import Image


from ..models import Article, Episode, Page, Program, StaticPage
from .mixins import ParentMixin
from .page import PageDetailView, PageListView, PageUpdateView

__all__ = ["ProgramPageDetailView", "ProgramDetailView", "ProgramPageListView", "ProgramUpdateView"]


class BaseProgramMixin:
    def get_program(self):
        return self.object

    def get_context_data(self, **kwargs):
        self.program = self.get_program()
        kwargs["program"] = self.program
        return super().get_context_data(**kwargs)


class ProgramDetailView(BaseProgramMixin, PageDetailView):
    model = Program

    def get_related_queryset(self):
        queryset = (
            self.get_queryset()
            .filter(category_id=self.object.category_id)
            .exclude(pk=self.object.pk)
            .published()
            .order_by("-pub_date")[:50]
        )
        return random.sample(list(queryset), min(len(queryset), self.related_count))

    def get_related_url(self):
        return reverse("program-list") + f"?category__id={self.object.category_id}"

    def get_context_data(self, **kwargs):
        episodes = Episode.objects.program(self.object).published().order_by("-pub_date")
        podcasts = episodes.with_podcasts()
        articles = Article.objects.parent(self.object).published().order_by("-pub_date")
        return super().get_context_data(
            articles=articles[: self.related_count],
            episodes=episodes[: self.related_count],
            podcasts=podcasts[: self.related_count],
            **kwargs,
        )

    def get_template_names(self):
        return super().get_template_names() + ["aircox/program_detail.html"]


# FIXME: forms should be put in aircox.forms
class ProgramForm(ModelForm):
    content = RichTextField()
    new_cover = ImageField(required=False)

    class Meta:
        model = Program
        fields = ["content"]

    def save(self, commit=True):
        file_obj = self.cleaned_data["new_cover"]
        if file_obj:
            obj, _ = Image.objects.get_or_create(original_filename=file_obj.name, file=file_obj)
            self.instance.cover = obj
        super().save(commit=commit)


class ProgramUpdateView(UserPassesTestMixin, BaseProgramMixin, PageUpdateView):
    model = Program
    form_class = ProgramForm

    # FIXME: ???
    def test_func(self):
        program = self.get_object()
        return self.request.user.has_perm("aircox.%s" % program.change_permission_codename)

    def get_success_url(self):
        return reverse("program-detail", kwargs={"slug": self.get_object().slug})


class ProgramListView(PageListView):
    model = Program
    attach_to_value = StaticPage.Target.PROGRAMS

    def get_queryset(self):
        return super().get_queryset().order_by("title")


# FIXME: not used
class ProgramPageDetailView(BaseProgramMixin, ParentMixin, PageDetailView):
    """Base view class for a page that is displayed as a program's child
    page."""

    parent_model = Program

    def get_program(self):
        self.parent = self.object.program
        return self.object.program


class ProgramPageListView(BaseProgramMixin, PageListView):
    model = Page
    parent_model = Program
    queryset = Page.objects.select_subclasses()

    def get_program(self):
        return self.parent

    def get_context_data(self, **kwargs):
        kwargs.setdefault("sidebar_url_parent", None)
        return super().get_context_data(**kwargs)
